@@ -35,27 +35,39 @@
 #include "menu_common.h"
 #include "menu_debug.h"
 #include "menu_drive.h"
+#include "menu_edit.h"
 #include "menu_ffmpeg.h"
 #include "menu_help.h"
+#include "menu_jam.h"
+#include "menu_joyport.h"
+#include "menu_media.h"
+#include "menu_monitor.h"
 #include "menu_network.h"
 #include "menu_petcart.h"
 #include "menu_pethw.h"
 #include "menu_printer.h"
 #include "menu_reset.h"
+#include "menu_sampler.h"
 #include "menu_screenshot.h"
 #include "menu_settings.h"
+#include "menu_sid.h"
 #include "menu_snapshot.h"
 #include "menu_sound.h"
 #include "menu_speed.h"
 #include "menu_tape.h"
 #include "menu_video.h"
 #include "petmem.h"
+#include "petui.h"
+#include "pet-resources.h"
 #include "resources.h"
 #include "ui.h"
+#include "uifonts.h"
 #include "uimenu.h"
 #include "vkbd.h"
 
-static const ui_menu_entry_t xpet_main_menu[] = {
+static UI_MENU_CALLBACK(pause_callback_wrapper);
+
+static ui_menu_entry_t xpet_main_menu[] = {
     { "Autostart image",
       MENU_ENTRY_DIALOG,
       autostart_callback,
@@ -88,14 +100,18 @@ static const ui_menu_entry_t xpet_main_menu[] = {
       MENU_ENTRY_SUBMENU,
       submenu_callback,
       (ui_callback_data_t)sound_output_menu },
+    { "Sampler settings",
+      MENU_ENTRY_SUBMENU,
+      submenu_callback,
+      (ui_callback_data_t)sampler_menu },
     { "Snapshot",
       MENU_ENTRY_SUBMENU,
       submenu_callback,
       (ui_callback_data_t)snapshot_menu },
-    { "Screenshot",
+    { "Save media file",
       MENU_ENTRY_SUBMENU,
       submenu_callback,
-      (ui_callback_data_t)screenshot_menu },
+      (ui_callback_data_t)media_menu },
     { "Speed settings",
       MENU_ENTRY_SUBMENU,
       submenu_callback,
@@ -104,6 +120,10 @@ static const ui_menu_entry_t xpet_main_menu[] = {
       MENU_ENTRY_SUBMENU,
       submenu_callback,
       (ui_callback_data_t)reset_menu },
+    { "Action on CPU JAM",
+      MENU_ENTRY_SUBMENU,
+      submenu_callback,
+      (ui_callback_data_t)jam_menu },
 #ifdef HAVE_NETWORK
     { "Network",
       MENU_ENTRY_SUBMENU,
@@ -111,19 +131,25 @@ static const ui_menu_entry_t xpet_main_menu[] = {
       (ui_callback_data_t)network_menu },
 #endif
     { "Pause",
+      MENU_ENTRY_OTHER_TOGGLE,
+      pause_callback_wrapper,
+      NULL },
+    /* Caution: index is hardcoded below */
+    { "Advance Frame",
       MENU_ENTRY_OTHER,
-      pause_callback,
+      advance_frame_callback,
       NULL },
     { "Monitor",
-      MENU_ENTRY_OTHER,
-      monitor_callback,
-      NULL },
+      MENU_ENTRY_SUBMENU,
+      submenu_callback,
+      (ui_callback_data_t)monitor_menu },
+    /* Caution: index is hardcoded below */
     { "Virtual keyboard",
       MENU_ENTRY_OTHER,
       vkbd_callback,
       NULL },
     { "Statusbar",
-      MENU_ENTRY_OTHER,
+      MENU_ENTRY_OTHER_TOGGLE,
       statusbar_callback,
       NULL },
 #ifdef DEBUG
@@ -140,6 +166,12 @@ static const ui_menu_entry_t xpet_main_menu[] = {
       MENU_ENTRY_SUBMENU,
       submenu_callback,
       (ui_callback_data_t)settings_manager_menu },
+#ifdef USE_SDLUI2
+    { "Edit",
+      MENU_ENTRY_SUBMENU,
+      submenu_callback,
+      (ui_callback_data_t)edit_menu },
+#endif
     { "Quit emulator",
       MENU_ENTRY_OTHER,
       quit_callback,
@@ -147,52 +179,89 @@ static const ui_menu_entry_t xpet_main_menu[] = {
     SDL_MENU_LIST_END
 };
 
-static BYTE *pet_font;
+#ifdef HAVE_NETWORK
+# define MENU_ADVANCE_FRAME_IDX      16
+# define MENU_VIRTUAL_KEYBOARD_IDX   18
+#else
+# define MENU_ADVANCE_FRAME_IDX      15
+# define MENU_VIRTUAL_KEYBOARD_IDX   17
+#endif
+static UI_MENU_CALLBACK(pause_callback_wrapper)
+{
+    xpet_main_menu[MENU_ADVANCE_FRAME_IDX].status = 
+        sdl_pause_state || !sdl_menu_state ? MENU_STATUS_ACTIVE : MENU_STATUS_INACTIVE;
+    xpet_main_menu[MENU_VIRTUAL_KEYBOARD_IDX].status =
+        sdl_pause_state ? MENU_STATUS_INACTIVE : MENU_STATUS_ACTIVE;
+    return pause_callback(activated, param);
+}
 
-void petui_set_menu_params(int index, menu_draw_t *menu_draw)
+/* FIXME: support all PET keyboards (see pet-resources.h) */
+
+static void petui_set_menu_params(int index, menu_draw_t *menu_draw)
 {
     static int old_keymap = -1;
-    int cols, keymap;
-
-    resources_get_int("VideoSize", &cols);
+    int cols = petmem_get_rom_columns();
+    int keymap;
 
     menu_draw->max_text_x = cols ? cols : 40;
-    menu_draw->extra_x = 32;
-    menu_draw->extra_y = (cols == 40) ? 40 : 28;
+    menu_draw->extra_x = 24;
+    menu_draw->extra_y = (cols == 40) ? 32 : 20;
 
-    resources_get_int("KeymapIndex", &keymap);
+    menu_draw->max_text_y = (cols == 40) ? 26 : 28;
+
+    resources_get_int("KeyboardType", &keymap);
 
-    keymap &= 2;
     if (keymap != old_keymap) {
-        if (keymap) {
+        if (keymap == KBD_TYPE_GRAPHICS_US) {
             sdl_vkbd_set_vkbd(&vkbd_pet_gr);
         } else {
             sdl_vkbd_set_vkbd(&vkbd_pet_uk);
         }
         old_keymap = keymap;
     }
+
+    /* CRTC */
+    menu_draw->color_front = menu_draw->color_default_front = 1;
+    menu_draw->color_back = menu_draw->color_default_back = 0;
+    menu_draw->color_cursor_back = 0;
+    menu_draw->color_cursor_revers = 1;
+    menu_draw->color_active_green = 1;
+    menu_draw->color_inactive_red = 1;
+    menu_draw->color_active_grey = 1;
+    menu_draw->color_inactive_grey = 1;
 }
 
+/** \brief  Pre-initialize the UI before the canvas window gets created
+ *
+ * \return  0 on success, -1 on failure
+ */
+int petui_init_early(void)
+{
+    return 0;
+}
+
+/** \brief  Initialize the UI
+ *
+ * \return  0 on success, -1 on failure
+ */
 int petui_init(void)
 {
-    int i, j;
 
 #ifdef SDL_DEBUG
-    fprintf(stderr,"%s\n",__func__);
+    fprintf(stderr, "%s\n", __func__);
 #endif
 
     sdl_ui_set_menu_params = petui_set_menu_params;
+    uijoyport_menu_create(0, 0, 1, 1, 0);
+    uisampler_menu_create();
+    uidrive_menu_create();
+    uikeyboard_menu_create();
+    uipalette_menu_create("Crtc", NULL);
+    uisid_menu_create();
+    uimedia_menu_create();
 
     sdl_ui_set_main_menu(xpet_main_menu);
-
-    pet_font=lib_malloc(8 * 256);
-    for (i = 0; i < 128; i++) {
-        for (j = 0; j < 8; j++) {
-            pet_font[(i * 8) + j] = mem_chargen_rom[(i * 16) + (256 * 16) + j];
-            pet_font[(i * 8) + (128 * 8) + j] = mem_chargen_rom[(i * 16) + j];
-        }
-    }
-    sdl_ui_set_menu_font(pet_font, 8, 8);
+    sdl_ui_crtc_font_init();
 
 #ifdef HAVE_FFMPEG
     sdl_menu_ffmpeg_init();
@@ -203,13 +272,18 @@ int petui_init(void)
 
 void petui_shutdown(void)
 {
+    uikeyboard_menu_shutdown();
+    uisid_menu_shutdown();
+    uipalette_menu_shutdown();
+    uijoyport_menu_shutdown();
+    uimedia_menu_shutdown();
 #ifdef SDL_DEBUG
-    fprintf(stderr,"%s\n",__func__);
+    fprintf(stderr, "%s\n", __func__);
 #endif
 
 #ifdef HAVE_FFMPEG
     sdl_menu_ffmpeg_shutdown();
 #endif
 
-    lib_free(pet_font);
+    sdl_ui_crtc_font_shutdown();
 }
