@@ -32,12 +32,14 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "joyport.h"
 #include "maincpu.h"
 #include "raster-changes.h"
 #include "types.h"
 #include "vic.h"
 #include "victypes.h"
 #include "vic-mem.h"
+#include "vic20.h"
 #include "vic20mem.h"
 #include "vic20sound.h"
 #include "viewport.h"
@@ -49,7 +51,7 @@
 
 /* VIC access functions. */
 
-void vic_store(WORD addr, BYTE value)
+void vic_store(uint16_t addr, uint8_t value)
 {
     addr &= 0xf;
     vic.regs[addr] = value;
@@ -217,50 +219,49 @@ void vic_store(WORD addr, BYTE value)
     }
 }
 
-BYTE vic_read(WORD addr)
+static inline unsigned vic_read_rasterline(void)
+{
+    unsigned ypos = VIC_RASTER_Y(maincpu_clk + vic.cycle_offset);
+
+    /* HACK: on NTSC, rasterline 0 seems to start 33 cycles late. when interlace
+     *       is enabled, this seems to happen only on the first half frame.
+     */
+    /* TODO: examine/verify what exactly happens, and in what cycles. the over-
+     *       all frame timing must be correct (as hardcoded timer values in
+     *       games work) so the only way it makes sense is that when rasterline 0
+     *       is "shorter", the last rasterline has to be "longer".
+     */
+    if ((ypos == 0) && (vic.cycles_per_line == VIC20_NTSC_CYCLES_PER_LINE)) {
+        if (VIC_RASTER_CYCLE(maincpu_clk + vic.cycle_offset) < 33) {
+            return vic.screen_height - 1; /* confirm this */
+        }
+    }
+    return ypos;
+}
+
+uint8_t vic_read(uint16_t addr)
 {
     addr &= 0xf;
 
     switch (addr) {
         case 3:
-            return ((VIC_RASTER_Y(maincpu_clk + vic.cycle_offset) & 1) << 7)
-                   | (vic.regs[3] & ~0x80);
+            return ((vic_read_rasterline() & 1) << 7) | (vic.regs[3] & ~0x80);
         case 4:
-            return VIC_RASTER_Y(maincpu_clk + vic.cycle_offset) >> 1;
+            return vic_read_rasterline() >> 1;
         case 6:
             return vic.light_pen.x;
         case 7:
             return vic.light_pen.y;
-#ifdef HAVE_MOUSE
         case 8:
-            if (_mouse_enabled) {
-                return mouse_get_x();
-            } else if (lightpen_enabled) {
-                return lightpen_read_button_x();
-            } else {
-                return 0xff;
-            }
-            break;
+            return read_joyport_potx();
         case 9:
-            if (_mouse_enabled) {
-                return mouse_get_y();
-            } else if (lightpen_enabled) {
-                return lightpen_read_button_y();
-            } else {
-                return 0xff;
-            }
-            break;
-#else
-        case 8:
-        case 9:
-            return 0xff;
-#endif
+            return read_joyport_poty();
         default:
             return vic.regs[addr];
     }
 }
 
-BYTE vic_peek(WORD addr)
+uint8_t vic_peek(uint16_t addr)
 {
     /* No side effects (unless mouse_get_* counts) */
     return vic_read(addr);
